Package: mgcv
Version: 1.8-23
Author: Simon Wood <simon.wood@r-project.org>
Maintainer: Simon Wood <simon.wood@r-project.org>
Title: Mixed GAM Computation Vehicle with Automatic Smoothness
        Estimation
Description: Generalized additive (mixed) models, some of their extensions and 
             other generalized ridge regression with multiple smoothing 
             parameter estimation by (Restricted) Marginal Likelihood, 
             Generalized Cross Validation and similar. Includes a gam() 
             function, a wide variety of smoothers, JAGS support and 
             distributions beyond the exponential family.
Priority: recommended
Depends: R (>= 2.14.0), nlme (>= 3.1-64)
Imports: methods, stats, graphics, Matrix
Suggests: splines, parallel, survival, MASS
LazyLoad: yes
ByteCompile: yes
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2018-01-08 15:15:43 UTC; sw283
Repository: CRAN
Date/Publication: 2018-01-15 00:23:36 UTC
